import { Slide } from './classes/slide';
import { AutomizerParams, AutomizerSummary, SourceSlideIdentifier } from './types/types';
import { IPresentationProps } from './interfaces/ipresentation-props';
import { PresTemplate } from './interfaces/pres-template';
import { RootPresTemplate } from './interfaces/root-pres-template';
import { TemplateInfo } from './types/xml-types';
/**
 * Automizer
 *
 * The basic class for `pptx-automizer` package.
 * This class will be exported as `Automizer` by `index.ts`.
 */
export default class Automizer implements IPresentationProps {
    rootTemplate: RootPresTemplate;
    /**
     * Templates  of automizer
     * @internal
     */
    templates: PresTemplate[];
    templateDir: string;
    outputDir: string;
    /**
     * Timer  of automizer
     * @internal
     */
    timer: number;
    params: AutomizerParams;
    /**
     * Creates an instance of `pptx-automizer`.
     * @param [params]
     */
    constructor(params: AutomizerParams);
    /**
  
     */
    presentation(): Promise<this>;
    /**
     * Load a pptx file and set it as root template.
     * @param location - Filename or path to the template. Will be prefixed with 'templateDir'
     * @returns Instance of Automizer
     */
    loadRoot(location: string): this;
    /**
     * Load a template pptx file.
     * @param location - Filename or path to the template. Will be prefixed with 'templateDir'
     * @param name - Optional: A short name for the template. If skipped, the template will be named by its location.
     * @returns Instance of Automizer
     */
    load(location: string, name?: string): this;
    /**
     * Loads a pptx file either as a root template as a template file.
     * A name can be specified to give templates an alias.
     * @param location
     * @param [name]
     * @returns template
     */
    private loadTemplate;
    /**
     * Parses all loaded templates and collects creationIds for slides and
     * elements. This will make finding templates and elements independent
     * from slide number and element name.
     * @returns Promise<TemplateInfo[]>
     */
    setCreationIds(): Promise<TemplateInfo[]>;
    /**
     * Determines whether template is root or default template.
     * @param template
     * @returns pres template
     */
    private isPresTemplate;
    /**
     * Add a slide from one of the imported templates by slide number or creationId.
     * @param name - Name or alias of the template; must have been loaded with `Automizer.load()`
     * @param slideIdentifier - Number or creationId of slide in template presentation
     * @param callback - Executed after slide was added. The newly created slide will be passed to the callback as first argument.
     * @returns Instance of Automizer
     */
    addSlide(name: string, slideIdentifier: SourceSlideIdentifier, callback?: (slide: Slide) => void): this;
    addMaster(name: string, masterNumber: number, callback?: (slide: Slide) => void): this;
    /**
     * Searches this.templates to find template by given name.
     * @internal
     * @param name Alias name if given to loaded template.
     * @returns template
     */
    getTemplate(name: string): PresTemplate;
    /**
     * Write all imports and modifications to a file.
     * @param location - Filename or path for the file. Will be prefixed with 'outputDir'
     * @returns summary object.
     */
    write(location: string): Promise<AutomizerSummary>;
    /**
     * Applies path prefix to given location string.
     * @param location path and/or filename
     * @param [type] template or output
     * @returns location
     */
    private getLocation;
}
