"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const slide_1 = require("./classes/slide");
const file_helper_1 = require("./helper/file-helper");
const template_1 = require("./classes/template");
const master_1 = require("./classes/master");
/**
 * Automizer
 *
 * The basic class for `pptx-automizer` package.
 * This class will be exported as `Automizer` by `index.ts`.
 */
class Automizer {
    /**
     * Creates an instance of `pptx-automizer`.
     * @param [params]
     */
    constructor(params) {
        this.templates = [];
        this.params = params;
        this.templateDir = (params === null || params === void 0 ? void 0 : params.templateDir) ? params.templateDir + '/' : '';
        this.outputDir = (params === null || params === void 0 ? void 0 : params.outputDir) ? params.outputDir + '/' : '';
        this.timer = Date.now();
        if (params.rootTemplate) {
            const location = this.getLocation(params.rootTemplate, 'template');
            this.rootTemplate = template_1.Template.import(location);
        }
        if (params.presTemplates) {
            this.params.presTemplates.forEach((file) => {
                const location = this.getLocation(file, 'template');
                const newTemplate = template_1.Template.import(location, file);
                this.templates.push(newTemplate);
            });
        }
    }
    /**
  
     */
    presentation() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (((_a = this.params) === null || _a === void 0 ? void 0 : _a.useCreationIds) === true) {
                yield this.setCreationIds();
            }
            return this;
        });
    }
    /**
     * Load a pptx file and set it as root template.
     * @param location - Filename or path to the template. Will be prefixed with 'templateDir'
     * @returns Instance of Automizer
     */
    loadRoot(location) {
        return this.loadTemplate(location);
    }
    /**
     * Load a template pptx file.
     * @param location - Filename or path to the template. Will be prefixed with 'templateDir'
     * @param name - Optional: A short name for the template. If skipped, the template will be named by its location.
     * @returns Instance of Automizer
     */
    load(location, name) {
        name = name === undefined ? location : name;
        return this.loadTemplate(location, name);
    }
    /**
     * Loads a pptx file either as a root template as a template file.
     * A name can be specified to give templates an alias.
     * @param location
     * @param [name]
     * @returns template
     */
    loadTemplate(location, name) {
        location = this.getLocation(location, 'template');
        const alreadyLoaded = this.templates.find(template => template.name === name);
        if (alreadyLoaded) {
            return this;
        }
        const newTemplate = template_1.Template.import(location, name);
        if (!this.isPresTemplate(newTemplate)) {
            this.rootTemplate = newTemplate;
        }
        else {
            this.templates.push(newTemplate);
        }
        return this;
    }
    /**
     * Parses all loaded templates and collects creationIds for slides and
     * elements. This will make finding templates and elements independent
     * from slide number and element name.
     * @returns Promise<TemplateInfo[]>
     */
    setCreationIds() {
        return __awaiter(this, void 0, void 0, function* () {
            const templateCreationId = [];
            for (const template of this.templates) {
                const creationIds = template.creationIds || (yield template.setCreationIds());
                templateCreationId.push({
                    name: template.name,
                    slides: creationIds,
                });
            }
            return templateCreationId;
        });
    }
    /**
     * Determines whether template is root or default template.
     * @param template
     * @returns pres template
     */
    isPresTemplate(template) {
        return 'name' in template;
    }
    /**
     * Add a slide from one of the imported templates by slide number or creationId.
     * @param name - Name or alias of the template; must have been loaded with `Automizer.load()`
     * @param slideIdentifier - Number or creationId of slide in template presentation
     * @param callback - Executed after slide was added. The newly created slide will be passed to the callback as first argument.
     * @returns Instance of Automizer
     */
    addSlide(name, slideIdentifier, callback) {
        if (this.rootTemplate === undefined) {
            throw new Error('You have to set a root template first.');
        }
        const template = this.getTemplate(name);
        const newSlide = new slide_1.Slide({
            presentation: this,
            template,
            slideIdentifier,
        });
        if (callback !== undefined) {
            newSlide.root = this;
            callback(newSlide);
        }
        this.rootTemplate.slides.push(newSlide);
        return this;
    }
    addMaster(name, masterNumber, callback) {
        const template = this.getTemplate(name);
        const newMaster = new master_1.Master({
            presentation: this,
            template,
            masterNumber,
        });
        // this.rootTemplate.slides.push(newMaster);
        return this;
    }
    /**
     * Searches this.templates to find template by given name.
     * @internal
     * @param name Alias name if given to loaded template.
     * @returns template
     */
    getTemplate(name) {
        const template = this.templates.find((t) => t.name === name);
        if (template === undefined) {
            throw new Error(`Template not found: ${name}`);
        }
        return template;
    }
    /**
     * Write all imports and modifications to a file.
     * @param location - Filename or path for the file. Will be prefixed with 'outputDir'
     * @returns summary object.
     */
    write(location) {
        return __awaiter(this, void 0, void 0, function* () {
            const rootArchive = yield this.rootTemplate.archive;
            for (const slide of this.rootTemplate.slides) {
                yield this.rootTemplate.appendSlide(slide);
            }
            const content = yield rootArchive.generateAsync({ type: 'nodebuffer' });
            return file_helper_1.FileHelper.writeOutputFile(this.getLocation(location, 'output'), content, this);
        });
    }
    /**
     * Applies path prefix to given location string.
     * @param location path and/or filename
     * @param [type] template or output
     * @returns location
     */
    getLocation(location, type) {
        switch (type) {
            case 'template':
                return this.templateDir + location;
            case 'output':
                return this.outputDir + location;
            default:
                return location;
        }
    }
}
exports.default = Automizer;
//# sourceMappingURL=automizer.js.map