"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Slide = void 0;
const file_helper_1 = require("../helper/file-helper");
const xml_helper_1 = require("../helper/xml-helper");
const element_type_1 = require("../enums/element-type");
const image_1 = require("../shapes/image");
const chart_1 = require("../shapes/chart");
const generic_1 = require("../shapes/generic");
class Slide {
    constructor(params) {
        this.sourceTemplate = params.template;
        this.sourceNumber = this.getSlideNumber(params.template, params.slideIdentifier);
        this.sourcePath = `ppt/slides/slide${this.sourceNumber}.xml`;
        this.relsPath = `ppt/slides/_rels/slide${this.sourceNumber}.xml.rels`;
        this.modifications = [];
        this.importElements = [];
    }
    /**
     * Try to convert a given slide's creationId to corresponding slide number.
     * Used if automizer is run with useCreationIds: true
     * @internal
     * @param PresTemplate
     * @slideNumber SourceSlideIdentifier
     * @returns number
     */
    getSlideNumber(template, slideIdentifier) {
        if (template.creationIds !== undefined) {
            const matchCreationId = template.creationIds.find((slideInfo) => slideInfo.id === Number(slideIdentifier));
            if (matchCreationId) {
                return matchCreationId.number;
            }
            throw ('Could not find slide number for creationId: ' + slideIdentifier + '@' + template.name);
        }
        return slideIdentifier;
    }
    /**
     * Appends slide
     * @internal
     * @param targetTemplate
     * @returns append
     */
    append(targetTemplate) {
        return __awaiter(this, void 0, void 0, function* () {
            this.targetTemplate = targetTemplate;
            this.targetArchive = yield targetTemplate.archive;
            this.targetNumber = targetTemplate.incrementCounter('slides');
            this.targetPath = `ppt/slides/slide${this.targetNumber}.xml`;
            this.targetRelsPath = `ppt/slides/_rels/slide${this.targetNumber}.xml.rels`;
            this.sourceArchive = yield this.sourceTemplate.archive;
            console.log('Appending slide ' + this.targetNumber);
            yield this.copySlideFiles();
            yield this.copyRelatedContent();
            yield this.addSlideToPresentation();
            const sourceNotesNumber = yield this.getSlideNoteSourceNumber();
            if (sourceNotesNumber) {
                yield this.copySlideNoteFiles(sourceNotesNumber);
                yield this.updateSlideNoteFile(sourceNotesNumber);
                yield this.appendNotesToContentType(this.targetArchive, this.targetNumber);
            }
            if (this.importElements.length) {
                yield this.importedSelectedElements();
            }
            yield this.applyModifications();
        });
    }
    /**
     * Modifys slide
     * @internal
     * @param callback
     */
    modify(callback) {
        this.modifications.push(callback);
    }
    /**
     * Adds slide to presentation
     * @internal
     * @returns slide to presentation
     */
    addSlideToPresentation() {
        return __awaiter(this, void 0, void 0, function* () {
            const relId = yield xml_helper_1.XmlHelper.getNextRelId(this.targetArchive, 'ppt/_rels/presentation.xml.rels');
            yield this.appendToSlideRel(this.targetArchive, relId, this.targetNumber);
            yield this.appendToSlideList(this.targetArchive, relId);
            yield this.appendSlideToContentType(this.targetArchive, this.targetNumber);
        });
    }
    /**
     * Select and modify a single element on an added slide.
     * @param {string} selector - Element's name on the slide.
     * Should be a unique string defined on the "Selection"-pane within ppt.
     * @param {ShapeModificationCallback | ShapeModificationCallback[]} callback - One or more callback functions to apply.
     * Depending on the shape type (e.g. chart or table), different arguments will be passed to the callback.
     */
    modifyElement(selector, callback) {
        const presName = this.sourceTemplate.name;
        const slideNumber = this.sourceNumber;
        return this.addElementToModificationsList(presName, slideNumber, selector, 'modify', callback);
    }
    /**
     * Select, insert and (optionally) modify a single element to a slide.
     * @param {string} presName - Filename or alias name of the template presentation.
     * Must have been importet with Automizer.load().
     * @param {number} slideNumber - Slide number within the specified template to search for the required element.
     * @param {ShapeModificationCallback | ShapeModificationCallback[]} callback - One or more callback functions to apply.
     * Depending on the shape type (e.g. chart or table), different arguments will be passed to the callback.
     */
    addElement(presName, slideNumber, selector, callback) {
        return this.addElementToModificationsList(presName, slideNumber, selector, 'append', callback);
    }
    /**
     * Adds element to modifications list
     * @internal
     * @param presName
     * @param slideNumber
     * @param selector
     * @param mode
     * @param [callback]
     * @returns element to modifications list
     */
    addElementToModificationsList(presName, slideNumber, selector, mode, callback) {
        this.importElements.push({
            presName,
            slideNumber,
            selector,
            mode,
            callback,
        });
        return this;
    }
    /**
     * Imported selected elements
     * @internal
     * @returns selected elements
     */
    importedSelectedElements() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const element of this.importElements) {
                const info = yield this.getElementInfo(element);
                switch (info.type) {
                    case element_type_1.ElementType.Chart:
                        yield new chart_1.Chart(info)[info.mode](this.targetTemplate, this.targetNumber);
                        break;
                    case element_type_1.ElementType.Image:
                        yield new image_1.Image(info)[info.mode](this.targetTemplate, this.targetNumber);
                        break;
                    case element_type_1.ElementType.Shape:
                        yield new generic_1.GenericShape(info)[info.mode](this.targetTemplate, this.targetNumber);
                        break;
                }
            }
        });
    }
    /**
     * Gets element info
     * @internal
     * @param importElement
     * @returns element info
     */
    getElementInfo(importElement) {
        return __awaiter(this, void 0, void 0, function* () {
            const template = this.root.getTemplate(importElement.presName);
            const slideNumber = importElement.mode === 'append'
                ? this.getSlideNumber(template, importElement.slideNumber)
                : importElement.slideNumber;
            const sourcePath = `ppt/slides/slide${slideNumber}.xml`;
            const sourceArchive = yield template.archive;
            const hasCreationId = template.creationIds !== undefined;
            const method = hasCreationId
                ? 'findByElementCreationId'
                : 'findByElementName';
            const sourceElement = yield xml_helper_1.XmlHelper[method](sourceArchive, sourcePath, importElement.selector);
            if (!sourceElement) {
                throw new Error(`Can't find ${importElement.selector} on slide ${slideNumber} in ${importElement.presName}`);
            }
            const appendElementParams = yield this.analyzeElement(sourceElement, sourceArchive, slideNumber);
            return {
                mode: importElement.mode,
                name: importElement.selector,
                hasCreationId: hasCreationId,
                sourceArchive,
                sourceSlideNumber: slideNumber,
                sourceElement,
                callback: importElement.callback,
                target: appendElementParams.target,
                type: appendElementParams.type,
            };
        });
    }
    /**
     * Analyzes element
     * @internal
     * @param sourceElement
     * @param sourceArchive
     * @param slideNumber
     * @returns element
     */
    analyzeElement(sourceElement, sourceArchive, slideNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const isChart = sourceElement.getElementsByTagName('c:chart');
            if (isChart.length) {
                return {
                    type: element_type_1.ElementType.Chart,
                    target: yield xml_helper_1.XmlHelper.getTargetByRelId(sourceArchive, slideNumber, sourceElement, 'chart'),
                };
            }
            const isImage = sourceElement.getElementsByTagName('p:nvPicPr');
            if (isImage.length) {
                return {
                    type: element_type_1.ElementType.Image,
                    target: yield xml_helper_1.XmlHelper.getTargetByRelId(sourceArchive, slideNumber, sourceElement, 'image'),
                };
            }
            return {
                type: element_type_1.ElementType.Shape,
            };
        });
    }
    /**
     * Applys modifications
     * @internal
     * @returns modifications
     */
    applyModifications() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const modification of this.modifications) {
                const xml = yield xml_helper_1.XmlHelper.getXmlFromArchive(this.targetArchive, this.targetPath);
                modification(xml);
                yield xml_helper_1.XmlHelper.writeXmlToArchive(this.targetArchive, this.targetPath, xml);
            }
        });
    }
    /**
     * Copys slide files
     * @internal
     */
    copySlideFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            yield file_helper_1.FileHelper.zipCopy(this.sourceArchive, `ppt/slides/slide${this.sourceNumber}.xml`, this.targetArchive, `ppt/slides/slide${this.targetNumber}.xml`);
            yield file_helper_1.FileHelper.zipCopy(this.sourceArchive, `ppt/slides/_rels/slide${this.sourceNumber}.xml.rels`, this.targetArchive, `ppt/slides/_rels/slide${this.targetNumber}.xml.rels`);
        });
    }
    /**
     * slideNote numbers differ from slide numbers if presentation
     * contains slides without notes. We need to find out
     * the proper enumeration of slideNote xml files.
     * @internal
     * @returns slide note file number
     */
    getSlideNoteSourceNumber() {
        return __awaiter(this, void 0, void 0, function* () {
            const targets = yield xml_helper_1.XmlHelper.getTargetsByRelationshipType(this.sourceArchive, `ppt/slides/_rels/slide${this.sourceNumber}.xml.rels`, 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/notesSlide');
            if (targets.length) {
                const targetNumber = targets[0].file
                    .replace('../notesSlides/notesSlide', '')
                    .replace('.xml', '');
                return Number(targetNumber);
            }
        });
    }
    /**
     * Copys slide note files
     * @internal
     * @returns slide note files
     */
    copySlideNoteFiles(sourceNotesNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            yield file_helper_1.FileHelper.zipCopy(this.sourceArchive, `ppt/notesSlides/notesSlide${sourceNotesNumber}.xml`, this.targetArchive, `ppt/notesSlides/notesSlide${this.targetNumber}.xml`);
            yield file_helper_1.FileHelper.zipCopy(this.sourceArchive, `ppt/notesSlides/_rels/notesSlide${sourceNotesNumber}.xml.rels`, this.targetArchive, `ppt/notesSlides/_rels/notesSlide${this.targetNumber}.xml.rels`);
        });
    }
    /**
     * Updates slide note file
     * @internal
     * @returns slide note file
     */
    updateSlideNoteFile(sourceNotesNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            yield xml_helper_1.XmlHelper.replaceAttribute(this.targetArchive, `ppt/notesSlides/_rels/notesSlide${this.targetNumber}.xml.rels`, 'Relationship', 'Target', `../slides/slide${this.sourceNumber}.xml`, `../slides/slide${this.targetNumber}.xml`);
            yield xml_helper_1.XmlHelper.replaceAttribute(this.targetArchive, `ppt/slides/_rels/slide${this.targetNumber}.xml.rels`, 'Relationship', 'Target', `../notesSlides/notesSlide${sourceNotesNumber}.xml`, `../notesSlides/notesSlide${this.targetNumber}.xml`);
        });
    }
    /**
     * Appends to slide rel
     * @internal
     * @param rootArchive
     * @param relId
     * @param slideCount
     * @returns to slide rel
     */
    appendToSlideRel(rootArchive, relId, slideCount) {
        return xml_helper_1.XmlHelper.append({
            archive: rootArchive,
            file: `ppt/_rels/presentation.xml.rels`,
            parent: (xml) => xml.getElementsByTagName('Relationships')[0],
            tag: 'Relationship',
            attributes: {
                Id: relId,
                Type: `http://schemas.openxmlformats.org/officeDocument/2006/relationships/slide`,
                Target: `slides/slide${slideCount}.xml`,
            },
        });
    }
    /**
     * Appends a new slide to slide list in presentation.xml.
     * If rootArchive has no slides, a new node will be created.
     * "id"-attribute of 'p:sldId'-element must be greater than 255.
     * @internal
     * @param rootArchive
     * @param relId
     * @returns to slide list
     */
    appendToSlideList(rootArchive, relId) {
        return xml_helper_1.XmlHelper.append({
            archive: rootArchive,
            file: `ppt/presentation.xml`,
            assert: (xml) => __awaiter(this, void 0, void 0, function* () {
                if (xml.getElementsByTagName('p:sldIdLst').length === 0) {
                    xml_helper_1.XmlHelper.insertAfter(xml.createElement('p:sldIdLst'), xml.getElementsByTagName('p:sldMasterIdLst')[0]);
                }
            }),
            parent: (xml) => xml.getElementsByTagName('p:sldIdLst')[0],
            tag: 'p:sldId',
            attributes: {
                id: (xml) => xml_helper_1.XmlHelper.getMaxId(xml.getElementsByTagName('p:sldId'), 'id', true, 256),
                'r:id': relId,
            },
        });
    }
    /**
     * Appends slide to content type
     * @internal
     * @param rootArchive
     * @param slideCount
     * @returns slide to content type
     */
    appendSlideToContentType(rootArchive, slideCount) {
        return xml_helper_1.XmlHelper.append(xml_helper_1.XmlHelper.createContentTypeChild(rootArchive, {
            PartName: `/ppt/slides/slide${slideCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.presentationml.slide+xml`,
        }));
    }
    /**
     * Appends notes to content type
     * @internal
     * @param rootArchive
     * @param slideCount
     * @returns notes to content type
     */
    appendNotesToContentType(rootArchive, slideCount) {
        return xml_helper_1.XmlHelper.append(xml_helper_1.XmlHelper.createContentTypeChild(rootArchive, {
            PartName: `/ppt/notesSlides/notesSlide${slideCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.presentationml.notesSlide+xml`,
        }));
    }
    /**
     * Copys related content
     * @internal
     * @returns related content
     */
    copyRelatedContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const charts = yield chart_1.Chart.getAllOnSlide(this.sourceArchive, this.relsPath);
            for (const chart of charts) {
                yield new chart_1.Chart({
                    mode: 'append',
                    target: chart,
                    sourceArchive: this.sourceArchive,
                    sourceSlideNumber: this.sourceNumber,
                }).modifyOnAddedSlide(this.targetTemplate, this.targetNumber);
            }
            const images = yield image_1.Image.getAllOnSlide(this.sourceArchive, this.relsPath);
            for (const image of images) {
                yield new image_1.Image({
                    mode: 'append',
                    target: image,
                    sourceArchive: this.sourceArchive,
                    sourceSlideNumber: this.sourceNumber,
                }).modifyOnAddedSlide(this.targetTemplate, this.targetNumber);
            }
        });
    }
}
exports.Slide = Slide;
//# sourceMappingURL=slide.js.map