"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XmlHelper = void 0;
const xmldom_1 = require("xmldom");
const file_helper_1 = require("./file-helper");
const constants_1 = require("../constants/constants");
const xml_pretty_print_1 = require("./xml-pretty-print");
class XmlHelper {
    static getXmlFromArchive(archive, file) {
        return __awaiter(this, void 0, void 0, function* () {
            const xmlDocument = (yield file_helper_1.FileHelper.extractFromArchive(archive, file));
            const dom = new xmldom_1.DOMParser();
            return dom.parseFromString(xmlDocument);
        });
    }
    static writeXmlToArchive(archive, file, xml) {
        return __awaiter(this, void 0, void 0, function* () {
            const s = new xmldom_1.XMLSerializer();
            const xmlBuffer = s.serializeToString(xml);
            return archive.file(file, xmlBuffer);
        });
    }
    static appendIf(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(element.archive, element.file);
            return element.clause !== undefined && !element.clause(xml)
                ? false
                : XmlHelper.append(element);
        });
    }
    static append(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(element.archive, element.file);
            const newElement = xml.createElement(element.tag);
            for (const attribute in element.attributes) {
                const value = element.attributes[attribute];
                newElement.setAttribute(attribute, typeof value === 'function' ? value(xml) : value);
            }
            if (element.assert) {
                element.assert(xml);
            }
            const parent = element.parent(xml);
            parent.appendChild(newElement);
            yield XmlHelper.writeXmlToArchive(element.archive, element.file, xml);
            return newElement;
        });
    }
    static getNextRelId(rootArchive, file) {
        return __awaiter(this, void 0, void 0, function* () {
            const presentationRelsXml = yield XmlHelper.getXmlFromArchive(rootArchive, file);
            const increment = (max) => 'rId' + max;
            const relationNodes = presentationRelsXml.documentElement.childNodes;
            const rid = XmlHelper.getMaxId(relationNodes, 'Id', true);
            return increment(rid);
        });
    }
    static getMaxId(rels, attribute, increment, minId) {
        let max = 0;
        for (const i in rels) {
            const rel = rels[i];
            if (rel.getAttribute !== undefined) {
                const id = Number(rel.getAttribute(attribute).replace('rId', ''));
                max = id > max ? id : max;
            }
        }
        switch (typeof increment) {
            case 'boolean':
                ++max;
                break;
        }
        if (max < minId) {
            return minId;
        }
        return max;
    }
    static getTargetsFromRelationships(archive, path, prefix, suffix) {
        return __awaiter(this, void 0, void 0, function* () {
            return XmlHelper.getRelationships(archive, path, (element, rels) => {
                const target = element.getAttribute('Target');
                if (target.indexOf(prefix) === 0) {
                    rels.push({
                        file: target,
                        rId: element.getAttribute('Id'),
                        number: Number(target.replace(prefix, '').replace(suffix || '.xml', '')),
                    });
                }
            });
        });
    }
    static getTargetsByRelationshipType(archive, path, type) {
        return __awaiter(this, void 0, void 0, function* () {
            return XmlHelper.getRelationships(archive, path, (element, rels) => {
                const target = element.getAttribute('Type');
                if (target === type) {
                    rels.push({
                        file: element.getAttribute('Target'),
                        rId: element.getAttribute('Id'),
                    });
                }
            });
        });
    }
    static getRelationships(archive, path, cb) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(archive, path);
            const relationships = xml.getElementsByTagName('Relationship');
            const rels = [];
            Object.keys(relationships)
                .map((key) => relationships[key])
                .filter((element) => element.getAttribute !== undefined)
                .forEach((element) => cb(element, rels));
            return rels;
        });
    }
    static findByAttribute(xml, tagName, attributeName, attributeValue) {
        const elements = xml.getElementsByTagName(tagName);
        for (const i in elements) {
            const element = elements[i];
            if (element.getAttribute !== undefined) {
                if (element.getAttribute(attributeName) === attributeValue) {
                    return true;
                }
            }
        }
        return false;
    }
    static replaceAttribute(archive, path, tagName, attributeName, attributeValue, replaceValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(archive, path);
            const elements = xml.getElementsByTagName(tagName);
            for (const i in elements) {
                const element = elements[i];
                if (element.getAttribute !== undefined &&
                    element.getAttribute(attributeName) === attributeValue) {
                    element.setAttribute(attributeName, replaceValue);
                }
            }
            return XmlHelper.writeXmlToArchive(archive, path, xml);
        });
    }
    static getTargetByRelId(archive, slideNumber, element, type) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = constants_1.TargetByRelIdMap[type];
            const sourceRid = element
                .getElementsByTagName(params.relRootTag)[0]
                .getAttribute(params.relAttribute);
            const relsPath = `ppt/slides/_rels/slide${slideNumber}.xml.rels`;
            const imageRels = yield XmlHelper.getTargetsFromRelationships(archive, relsPath, params.prefix, params.expression);
            const target = imageRels.find((rel) => rel.rId === sourceRid);
            return target;
        });
    }
    static findByElementCreationId(archive, path, creationId) {
        return __awaiter(this, void 0, void 0, function* () {
            const slideXml = yield XmlHelper.getXmlFromArchive(archive, path);
            return XmlHelper.findByCreationId(slideXml, creationId);
        });
    }
    static findByElementName(archive, path, name) {
        return __awaiter(this, void 0, void 0, function* () {
            const slideXml = yield XmlHelper.getXmlFromArchive(archive, path);
            return XmlHelper.findByName(slideXml, name);
        });
    }
    static findByName(doc, name) {
        const names = doc.getElementsByTagName('p:cNvPr');
        for (const i in names) {
            if (names[i].getAttribute && names[i].getAttribute('name') === name) {
                return names[i].parentNode.parentNode;
            }
        }
        return null;
    }
    static findByCreationId(doc, creationId) {
        const creationIds = doc.getElementsByTagName('a16:creationId');
        for (const i in creationIds) {
            if (creationIds[i].getAttribute &&
                creationIds[i].getAttribute('id') === creationId) {
                return creationIds[i].parentNode.parentNode.parentNode.parentNode
                    .parentNode;
            }
        }
        return null;
    }
    static findByAttributeValue(nodes, attributeName, attributeValue) {
        for (const i in nodes) {
            const node = nodes[i];
            if (node.getAttribute &&
                node.getAttribute(attributeName) === attributeValue) {
                return node;
            }
        }
        return null;
    }
    static createContentTypeChild(archive, attributes) {
        return {
            archive,
            file: `[Content_Types].xml`,
            parent: (xml) => xml.getElementsByTagName('Types')[0],
            tag: 'Override',
            attributes,
        };
    }
    static createRelationshipChild(archive, targetRelFile, attributes) {
        return {
            archive,
            file: targetRelFile,
            parent: (xml) => xml.getElementsByTagName('Relationships')[0],
            tag: 'Relationship',
            attributes,
        };
    }
    static appendSharedString(sharedStrings, stringValue) {
        const strings = sharedStrings.getElementsByTagName('sst')[0];
        const newLabel = sharedStrings.createTextNode(stringValue);
        const newText = sharedStrings.createElement('t');
        newText.appendChild(newLabel);
        const newString = sharedStrings.createElement('si');
        newString.appendChild(newText);
        strings.appendChild(newString);
        return strings.getElementsByTagName('si').length - 1;
    }
    static insertAfter(newNode, referenceNode) {
        referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
    }
    static sliceCollection(collection, length) {
        for (let i = collection.length; i > length; i--) {
            const toRemove = collection[i - 1];
            toRemove.parentNode.removeChild(toRemove);
        }
    }
    static dump(element) {
        const s = new xmldom_1.XMLSerializer();
        const xmlBuffer = s.serializeToString(element);
        const p = new xml_pretty_print_1.XmlPrettyPrint(xmlBuffer);
        p.dump();
    }
}
exports.XmlHelper = XmlHelper;
//# sourceMappingURL=xml-helper.js.map